#!/bin/bash
#
# Builds a .deb file, hopefully suitable for
# installation on .deb 
# Builds a .tar.bz2 file for Enchanting,
# suitable for distributing for Linux computers

# Make sure fpm ('Effing Package Management') is installed.
command -v fpm >/dev/null 2>&1 || { echo "Building a .deb requires 'fpm' and it is not installed or not on the path.  Try 'sudo gem install fpm' or run 'gem env' and add its paths to the system path." >&2; exit 1; }

if [ $# -ne "1" ]; then
        echo "Usage: `basename $0` versionNumber"
        echo "   ex: `basename $0` 0.0.3"
        exit 1
fi

if [[ $EUID -ne 0 ]]; then
   echo "This script must be run as root (so that the generated files belong to the root user)." 1>&2
   exit 1
fi

DIRNAME="Enchanting-$1"

pushd ../..
pwd
mkdir -p "Installers/Linux/$DIRNAME/usr/share/enchanting"
bash copyforrelease.sh "Installers/Linux/$DIRNAME/usr/share/enchanting" --verbose --exclude-from Installers/Linux/omit-from-linux-release
popd

rsync -a Skeleton/ "$DIRNAME" --verbose


# All files should be owned by root:root 
chown -R 0:0 "$DIRNAME"

echo "Building package"

fpm -s dir -t deb -a all -n enchanting -v "$1" -d java6-sdk -d scratch -d libusb-0.1-4 -d bluez -m "<clinton.blackmore+enchanting@gmail.com>" --description "Enchanting is a tool for programming LEGO Mindstorms NXT robots" --url "http://enchanting.robotclub.ab.ca" --after-install deb_postinstall.sh --before-remove deb_preremove.sh --license ../../License.txt -C "$DIRNAME" .


# 'bluez' is needed if you want to use bluetooth
# However, I assume that anyone who wants to use bluetooth will install the
# package separately; it isn't a strict dependency.

#sudo dpkg --install myfile.deb
#sudo apt-get --fix-broken install

